/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2024/08/01
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

/*
 *@Note
 *task1 and task2 alternate printing
 */

#include "debug.h"
#include "CH392INC.h"
#include "CH392.H"
#include "FreeRTOS.h"
#include "semphr.h"
#include "portmacro.h"
#include "task.h"
#include "event_groups.h"

/* Global Variable */
TaskHandle_t Ch392_init_Task_Handler;
TaskHandle_t CH392_InterruptPolling_Task_Handler;
TaskHandle_t CH392_Socket0_TCPclient_Loopback_Task_Handler;
TaskHandle_t CH392_Socket1_UDPServer_broadcast_Task_Handler;

SemaphoreHandle_t xCH392cmd_Mutex;
EventGroupHandle_t xCH392_Stauts;
EventGroupHandle_t xCH392_Socket_Stauts[8];

#define Event_CH392_Ready (1 << 0)
#define Event_CH392_Link_SUC (1 << 1)

#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
#include "CH392UART.h"
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
#include "CH392SPI_HW.h"
#else
#error "Please Select Correct Communication Interface "
#endif

/**********************************************************************************/
/* Include command file */
#include "CH392CMD.h"

/* Common variable definition */
UINT8 MyBuffer[1536];        /* data buffer */
struct _SOCK_INF SockInf[2]; /* Save the socket information */
struct _CH392_SYS CH392Inf;  /* Save the CH392 information */

/* CH392 Related definition */
const UINT8 CH392IPAddr[4] = {192, 168, 1, 101}; /* CH392 IP  */
const UINT8 CH392GWIPAddr[4] = {192, 168, 1, 1}; /* CH392 gateway */
const UINT8 CH392IPMask[4] = {255, 255, 255, 0}; /* CH392 mask */

/* Socket Related definition*/
const UINT8 Socket0DesIP[4] = {192, 168, 1, 123}; /* Socket 0 des IP */
const UINT16 Socket0SourPort = 1000;              /* Socket 0 sour port */
const UINT16 Socket0DesPort = 1000;               /* Socket 0 des port */

/* Socket Related definition*/
const UINT8 Socket1DesIP[4] = {255, 255, 255, 255}; /* Socket 0 des IP */
const UINT16 Socket1SourPort = 2000;                /* Socket 0 sour port */
const UINT16 Socket1DesPort = 2000;                 /* Socket 0 sour port */

/**********************************************************************************
 * Function Name  : mStopIfError
 * Description    : Display error code, and stop
 * Input          : iError
 * Output         : None
 * Return         : None
 **********************************************************************************/
void mStopIfError(UINT8 iError)
{
    if (iError == CMD_ERR_SUCCESS)
        return;                                /* Success */
    printf("Error: %02X\r\n", (UINT16)iError); /* printf error */
    while (1)
    {
        vTaskDelay(200);
    }
}

/**********************************************************************************
 * Function Name  : InitCH392InfParam
 * Description    : Initialize CH392Inf parameters
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void InitCH392InfParam(void)
{
    memset(&CH392Inf, 0, sizeof(CH392Inf));                          /* Clear all CH392Inf to zero */
    memcpy(CH392Inf.IPAddr, CH392IPAddr, sizeof(CH392IPAddr));       /* Enter the IP address in the CH392Inf file */
    memcpy(CH392Inf.GWIPAddr, CH392GWIPAddr, sizeof(CH392GWIPAddr)); /* Enter the gateway IP address in the CH392Inf file */
    memcpy(CH392Inf.MASKAddr, CH392IPMask, sizeof(CH392IPMask));     /* Enter the mask IP address in the CH392Inf file */
}

/**********************************************************************************
 * Function Name  : CH392Init
 * Description    : Configure IP, GWIP, and MAC for the CH392, and initialize the CH392
 * Input          : None
 * Output         : None
 * Return         : Function execution result
 **********************************************************************************/
UINT8 CH392Init(void)
{
    UINT8 i;

    i = CH392CMDCheckExist(0x65);
    if (i != 0x9a)
        return CH392_ERR_UNKNOW;            /* Test cmd , return 0XFA if it fails*/
                                            /* The value 0XFA is usually a hardware error or incorrect read/write timing */
    CH392CMDSetIPAddr(CH392Inf.IPAddr);     /* Set CH392 IP */
    CH392CMDSetGWIPAddr(CH392Inf.GWIPAddr); /* Set GW IP */
    CH392CMDSetMASKAddr(CH392Inf.MASKAddr); /* Set MASK */
    i = CH392CMDInitCH392();                /* initialize the CH392 */
    return i;
}

/**********************************************************************************
 * Function Name  : InitSocketParam
 * Description    : Initialize Socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void InitSocket0Param(void)
{
    memset(&SockInf[0], 0, sizeof(SockInf[0]));                    /* Clear all SockInf to zero*/
    memcpy(SockInf[0].IPAddr, Socket0DesIP, sizeof(Socket0DesIP)); /* Enter the DESIP */
    SockInf[0].DesPort = Socket0DesPort;                           /* DesPort */
    SockInf[0].SourPort = Socket0SourPort;                         /* SourPort */
    SockInf[0].ProtoType = PROTO_TYPE_TCP;                         /* TCP mode*/
}

/**********************************************************************************
 * Function Name  : CH392SocketInitOpen
 * Description    : Set CH392 socket parameters to initialize and open the socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392Socket0InitOpen(void)
{
    UINT8 i;
    CH392CMDSetSocketDesIP(0, SockInf[0].IPAddr);       /* set socket 0 des IP */
    CH392CMDSetSocketProtType(0, SockInf[0].ProtoType); /* set socket 0 type */
    CH392CMDSetSocketDesPort(0, SockInf[0].DesPort);    /* set socket 0 des port */
    CH392CMDSetSocketSourPort(0, SockInf[0].SourPort);  /* set socket 0 sour port */
    i = CH392CMDOpenSocket(0);                          /* open socket 0 */
    mStopIfError(i);                                    /* check the result */
}

/**********************************************************************************
 * Function Name  : InitSocketParam
 * Description    : Initialize Socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void InitSocket1Param(void)
{
    memset(&SockInf[1], 0, sizeof(SockInf[1]));                    /* Clear all SockInf to zero*/
    memcpy(SockInf[1].IPAddr, Socket1DesIP, sizeof(Socket0DesIP)); /* Enter the DESIP */
    SockInf[1].SourPort = Socket1SourPort;                         /* SourPort */
    SockInf[1].DesPort = Socket1DesPort;                           /* DesPort */
    SockInf[1].ProtoType = PROTO_TYPE_UDP;                         /* UDP mode*/
}

/**********************************************************************************
 * Function Name  : CH392SocketInitOpen
 * Description    : Set CH392 socket parameters to initialize and open the socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392Socket1InitOpen(void)
{
    UINT8 i;

    CH392CMDSetSocketDesIP(1, SockInf[1].IPAddr);       /* set socket 1 des IP */
    CH392CMDSetSocketProtType(1, SockInf[1].ProtoType); /* set socket 1 type */
    CH392CMDSetSocketDesPort(1, SockInf[1].DesPort);    /* set socket 1 des port */
    CH392CMDSetSocketSourPort(1, SockInf[1].SourPort);  /* set socket 1 sour port */
    i = CH392CMDOpenSocket(1);                          /* open socket 1 */
    mStopIfError(i);                                    /* check the result */
}

/**********************************************************************************
 * Function Name  : Gpio_Init
 * Description    : initialize GPIO
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void Gpio_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    // 392Interrupt
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    // 392Rset
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);
}

/**********************************************************************************
 * Function Name  : CH392_Init_Task
 * Description    : initialize CH392
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392_Init_Task()
{
    UINT8 i;
    CH392Reset();
    vTaskDelay(200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE)
    CH392SetUartBaudRate(921600);
#endif
    printf("CH392 initialize start\r\n");
    i = CH392CMDGetVer();
    printf("CH392VER : %2x\r\n", (UINT16)i);
    InitCH392InfParam(); /* Initialize CH392Inf parameters */
    i = CH392Init();     /* Initialize the CH392 */
    mStopIfError(i);
    xEventGroupSetBits(xCH392_Stauts, Event_CH392_Ready);
    printf("CH392 initialize success\r\n");
    vTaskDelete(NULL);
}

/**********************************************************************************
 * Function Name  : CH392_InterruptPolling_Task
 * Description    : CH392 Global interrupt function
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392_InterruptPolling_Task()
{
    xEventGroupWaitBits(xCH392_Stauts, Event_CH392_Ready, pdFALSE, pdFALSE, portMAX_DELAY);
    while (1)
    {
        vTaskDelay(1);
        if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1) == 0)
        {
            UINT16 init_status;

#if CHIP == CH392T
            init_status = CH392CMDGetGlobIntStatus_392T();
#elif CHIP == CH392F
            init_status = CH392CMDGetGlobIntStatus_392F();
#endif
            if (init_status & GINT_STAT_UNREACH) /* Unreachable interrupt,to read unreachable information */
            {
            }
            if (init_status & GINT_STAT_IP_CONFLI) /* IP address conflict interrupted,advice to change the IP address of the CH392 again and initialize the CH392 */
            {
            }
            if (init_status & GINT_STAT_PHY_CHANGE) /* PHY change interrupt */
            {
                if (CH392CMDGetPHYStatus() != 1)
                {
                    printf("PHY Link Suc\r\n");
                    xEventGroupSetBits(xCH392_Stauts, Event_CH392_Link_SUC);
                }
                else
                {
                    printf("PHY Link Down\r\n");
                    xEventGroupClearBits(xCH392_Stauts, Event_CH392_Link_SUC);
                }
            }
            if (init_status & GINT_STAT_DHCP)
            {
            }
            if (init_status & GINT_STAT_SOCK0)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[0], CH392CMDGetSocketInt(0));
            }
            if (init_status & GINT_STAT_SOCK1)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[1], CH392CMDGetSocketInt(1));
            }
            if (init_status & GINT_STAT_SOCK2)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[2], CH392CMDGetSocketInt(2));
            }
            if (init_status & GINT_STAT_SOCK3)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[3], CH392CMDGetSocketInt(3));
            }
#if CHIP == CH392T /* The 392T uses eight sockets by default and a maximum of 32 sockets. For details, see the 392 manual */
            if (init_status & GINT_STAT_SOCK4)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[4], CH392CMDGetSocketInt(4));
            }
            if (init_status & GINT_STAT_SOCK5)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[5], CH392CMDGetSocketInt(5));
            }
            if (init_status & GINT_STAT_SOCK6)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[6], CH392CMDGetSocketInt(6));
            }
            if (init_status & GINT_STAT_SOCK7)
            {
                xEventGroupSetBits(xCH392_Socket_Stauts[7], CH392CMDGetSocketInt(7));
            }
#endif
        }
    }
}

/**********************************************************************************
 * Function Name  : CH392_Socket0_TCPclient_Loopback_Task
 * Description    : initialize CH392
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392_Socket0_TCPclient_Loopback_Task()
{
    u8 i = 0;
    u16 totallen = 0, len = 0;
    // Wait for CH392 initialization to complete
    xEventGroupWaitBits(xCH392_Stauts, Event_CH392_Ready, pdFALSE, pdFALSE, portMAX_DELAY);
    printf("Socket0 create\n");
    InitSocket0Param();
    CH392Socket0InitOpen();

    xEventGroupWaitBits(xCH392_Stauts, Event_CH392_Link_SUC, pdFALSE, pdFALSE, portMAX_DELAY);
    i = CH392CMDTCPConnect(0);
    mStopIfError(i);
    printf("Socket0 create SUC\n");
    while (1)
    {
        xEventGroupWaitBits(xCH392_Socket_Stauts[0], SINT_STAT_RECV, pdTRUE, pdTRUE, portMAX_DELAY);
        totallen = CH392CMDGetRecvLength(0); /* Gets the length of data in the current buffer */
        printf("Socket0 receive\n");
        while (totallen > 0)
        {
            /*The length of the sent data packet cannot be greater than 1536,
            so only data with a length less than 1536 is read and sent at one time.
            In practical use, the length of received data depends solely on the size of the receive buffer.*/
            if (totallen > 1536)
            {
                len = 1536;
            }
            else
            {
                len = totallen;
            }
            totallen -= len;
            CH392CMDGetRecvData(0, len, MyBuffer);
            while (CH392CMDGetIdleSendQueue() == 0)
            {
                vTaskDelay(1);
            };
            CH392CMDSendData(0, MyBuffer, len);
        }
        vTaskDelay(1);
    }
}

/**********************************************************************************
 * Function Name  : CH392_Socket0_TCPclient_Loopback_Task
 * Description    : initialize CH392
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392_Socket1_UDPServer_broadcast_Task()
{
    // Wait for CH392 initialization to complete
    xEventGroupWaitBits(xCH392_Stauts, Event_CH392_Ready, pdFALSE, pdFALSE, portMAX_DELAY);
    printf("Socket1 create\n");
    InitSocket1Param();
    CH392Socket1InitOpen();
    printf("Socket1 create SUC\n");
    while (1)
    {
        vTaskDelay(500);
        xEventGroupWaitBits(xCH392_Socket_Stauts[1], SINT_STAT_SENBUF_FREE, pdTRUE, pdTRUE, portMAX_DELAY);
        printf("Socket1 send\n");
        CH392CMDSendData(1, MyBuffer, 100);
    }
}

/*******************************************************************************
 * Function Name  : main
 * Description    : Main program.
 * Input          : None
 * Return         : None
 *******************************************************************************/
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    printf("SystemClk:%d\r\n", SystemCoreClock);
    printf("FreeRTOS Kernel Version:%s\r\n", tskKERNEL_VERSION_NUMBER);

#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
    CH392_UARTPort_Init();
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
    CH392_SPIPort_Init();
#endif
    Gpio_Init();

    xCH392cmd_Mutex = xSemaphoreCreateMutex();
    xCH392_Stauts = xEventGroupCreate();
    for (u8 val = 0; val < 8; val++)
    {
        xCH392_Socket_Stauts[val] = xEventGroupCreate();
        /*
        If using the SINT_STAT_SENBUF_FREE interrupt as the condition for determining whether to send a data packet,
        the socket interrupt value can be set to SINT_STAT_SENBUF_FREE during initialization,
        as there will be no send buffer idle interrupt before any data packet is sent.
        */
        xEventGroupSetBits(xCH392_Socket_Stauts[val], SINT_STAT_SENBUF_FREE);
    }
    /* create two task */
    xTaskCreate((TaskFunction_t)CH392_Init_Task,
                (const char *)"CH392_Init",
                (uint16_t)128,
                (void *)NULL,
                (UBaseType_t)6,
                (TaskHandle_t *)&Ch392_init_Task_Handler);

    xTaskCreate((TaskFunction_t)CH392_InterruptPolling_Task,
                (const char *)"CH392_InterruptPolling",
                (uint16_t)128,
                (void *)NULL,
                (UBaseType_t)1,
                (TaskHandle_t *)&CH392_InterruptPolling_Task_Handler);

    xTaskCreate((TaskFunction_t)CH392_Socket0_TCPclient_Loopback_Task,
                (const char *)"CH392_Socket0_TCPclient_Loopback",
                (uint16_t)128,
                (void *)NULL,
                (UBaseType_t)4,
                (TaskHandle_t *)&CH392_Socket0_TCPclient_Loopback_Task_Handler);

    xTaskCreate((TaskFunction_t)CH392_Socket1_UDPServer_broadcast_Task,
                (const char *)"CH392_Socket1_UDPServer_broadcast",
                (uint16_t)128,
                (void *)NULL,
                (UBaseType_t)3,
                (TaskHandle_t *)&CH392_Socket1_UDPServer_broadcast_Task_Handler);

    vTaskStartScheduler();

    while (1)
    {
        printf("shouldn't run at here!!\n");
    }
}
