/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.1.0
 * Date               : 2024/08/01
 * Description        : Main program body.
 *******************************************************************************/
#include "debug.h"
#include "CH392INC.h"
#include "CH392.H"

#define Query392Interrupt() GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1)

#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
#include "CH392UART.h"
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
#include "CH392SPI_HW.h"
#else
#error "Please Select Correct Communication Interface "
#endif

/**********************************************************************************/
/* Include command file */
#include "CH392CMD.h"

/* Common variable definition */
UINT8 MyBuffer[4096];        /* data buffer */
struct _SOCK_INF SockInf[4]; /* Save the socket information */
struct _CH392_SYS CH392Inf;  /* Save the CH392 information */

/* CH392 Related definition */
const UINT8 CH392IPAddr[4] = {192, 168, 1, 101}; /* CH392 IP  */
const UINT8 CH392GWIPAddr[4] = {192, 168, 1, 1}; /* CH392 gateway */
const UINT8 CH392IPMask[4] = {255, 255, 255, 0}; /* CH392 mask */

/* Socket Related definition*/
const UINT16 Socket0SourPort = 1000; /* Socket 0 sour port */

/*
If using the SINT_STAT_SENBUF_FREE interrupt as the condition for determining whether to send a data packet,
the socket interrupt value can be set to SINT_STAT_SENBUF_FREE during initialization,
as there will be no send buffer idle interrupt before any data packet is sent.
*/
volatile UINT8 sock_int_status[8] = {
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
    SINT_STAT_SENBUF_FREE,
};

/**********************************************************************************
 * Function Name  : mStopIfError
 * Description    : Display error code, and stop
 * Input          : iError
 * Output         : None
 * Return         : None
 **********************************************************************************/
void mStopIfError(UINT8 iError)
{
    if (iError == CMD_ERR_SUCCESS)
        return;                                /* Success */
    printf("Error: %02X\r\n", (UINT16)iError); /* printf error */
    while (1)
    {
        Delay_Ms(200);
        Delay_Ms(200);
    }
}

/**********************************************************************************
 * Function Name  : InitCH392InfParam
 * Description    : Initialize CH392Inf parameters
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void InitCH392InfParam(void)
{
    memset(&CH392Inf, 0, sizeof(CH392Inf));                          /* Clear all CH392Inf to zero */
    memcpy(CH392Inf.IPAddr, CH392IPAddr, sizeof(CH392IPAddr));       /* Enter the IP address in the CH392Inf file */
    memcpy(CH392Inf.GWIPAddr, CH392GWIPAddr, sizeof(CH392GWIPAddr)); /* Enter the gateway IP address in the CH392Inf file */
    memcpy(CH392Inf.MASKAddr, CH392IPMask, sizeof(CH392IPMask));     /* Enter the mask IP address in the CH392Inf file */
}

/**********************************************************************************
 * Function Name  : InitSocketParam
 * Description    : Initialize Socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void InitSocketParam(void)
{
    memset(&SockInf[0], 0, sizeof(SockInf[0])); /* Clear all SockInf[0] to zero*/
    SockInf[0].SourPort = Socket0SourPort;      /* SourPort */
    SockInf[0].ProtoType = PROTO_TYPE_TCP;      /* TCP mode*/

    memset(&SockInf[1], 0, sizeof(SockInf[1])); /* Clear all SockInf[1] to zero*/
    SockInf[1].SourPort = Socket0SourPort;      /* SourPort */
    SockInf[1].ProtoType = PROTO_TYPE_TCP;      /* TCP mode*/
}

/**********************************************************************************
 * Function Name  : CH392SocketInitOpen
 * Description    : Set CH392 socket parameters to initialize and open the socket
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392SocketInitOpen(void)
{
    UINT8 i;

    CH392CMDSetSocketProtType(0, SockInf[0].ProtoType); /* set socket 0 type */
    CH392CMDSetSocketSourPort(0, SockInf[0].SourPort);  /* set socket 0 sour port */
    i = CH392CMDOpenSocket(0);                          /* open socket 0 */
    mStopIfError(i);                                    /* check the result */
    i = CH392CMDTCPListen(0);
    mStopIfError(i);

    CH392CMDSetSocketProtType(1, SockInf[1].ProtoType); /* set socket 1 type */
    CH392CMDSetSocketSourPort(1, SockInf[1].SourPort);  /* set socket 1 sour port */
}

/**********************************************************************************
 * Function Name  : CH392SocketInterrupt
 * Description    : CH392 socket interrupt, called in global interrupt
 * Input          : sockindex
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392SocketInterrupt(UINT8 sockindex)
{

    UINT16 len, totallen;


    sock_int_status[sockindex] |= CH392CMDGetSocketInt(sockindex); /* Gets the socket interrupt status */

    if (sock_int_status[sockindex] & SINT_STAT_RECV) /* Receive interruption */
    {
        sock_int_status[sockindex] &= ~SINT_STAT_RECV;
        totallen = CH392CMDGetRecvLength(sockindex); /* Gets the length of data in the current buffer */
//        printf("SINT_STAT_RECV\r\n",totallen);
        while (totallen > 0)
        {
            /*The length of the sent data packet cannot be greater than 1536,
            so only data with a length less than 1536 is read and sent at one time.
            In practical use, the length of received data depends solely on the size of the receive buffer.*/
            if (totallen > 1536)
            {
                len = 1536;
            }
            else
            {
                len = totallen;
            }
            totallen -= len;
            CH392CMDGetRecvData(sockindex, len, MyBuffer);
            /*
            This code snippet is not necessary in certain cases.
            It should be used in situations where data packets are being sent exceptionally frequently
            to verify if the sending conditions are met.
            */
#if SEND_CONDITION == SEND_QUEUE_IS_IDLE
            while (CH392CMDGetIdleSendQueue() == 0)
            {
                printf("Send Idle Queue = 0\r\n");
                Delay_Us(100);
            };
#elif SEND_CONDITION == SEND_BUFFER_IS_NOT_EMPTY
            while ((sock_int_status[sockindex] & SINT_STAT_SENBUF_FREE) == 0)
            {
                Delay_Us(100);
                sock_int_status[sockindex] |= CH392CMDGetSocketInt(sockindex);
            };
            sock_int_status[sockindex] &= ~SINT_STAT_SENBUF_FREE;
#endif
            CH392CMDSendData(sockindex, MyBuffer, len);
        }

    }
    if (sock_int_status[sockindex] & SINT_STAT_SENBUF_FREE) /* The send buffer is free and can continue writing data to be sent */
    {
/*
If SINT_STAT_SENBUF_FREE is used as the condition for sending packets,
then the interrupt status can be cleared during packet transmission queries.
*/
#if SEND_CONDITION != SEND_BUFFER_IS_NOT_EMPTY
        sock_int_status[sockindex] &= ~SINT_STAT_SENBUF_FREE;
#endif
    }
    if (sock_int_status[sockindex] & SINT_STAT_SEND_OK) /* Send completion interrupt */
    {
        sock_int_status[sockindex] &= ~SINT_STAT_SEND_OK;
    }
    if (sock_int_status[sockindex] & SINT_STAT_CONNECT) /* The connection is interrupted, valid only in TCP mode */
    {
        sock_int_status[sockindex] &= ~SINT_STAT_CONNECT;
        printf("SINT_STAT_CONNECT\r\n");
    }
    if (sock_int_status[sockindex] & SINT_STAT_DISCONNECT) /* Disconnect interrupt, valid only in TCP mode */
    {
        sock_int_status[sockindex] &= ~SINT_STAT_DISCONNECT;
        printf("SINT_STAT_DISCONNECT \r\n");
    }
    if (sock_int_status[sockindex] & SINT_STAT_TIM_OUT) /* Timeout interrupt, valid only in TCP mode */
    {
        sock_int_status[sockindex] &= ~SINT_STAT_TIM_OUT;
        printf("SINT_STAT_TIM_OUT\r\n");
    }
}

/**********************************************************************************
 * Function Name  : CH392GlobalInterrupt
 * Description    : CH392 Global interrupt function
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392GlobalInterrupt(void)
{
    UINT16 init_status;

#if CHIP == CH392T
    init_status = CH392CMDGetGlobIntStatus_392T();
#elif CHIP == CH392F
    init_status = CH392CMDGetGlobIntStatus_392F();
#endif

    if (init_status & GINT_STAT_UNREACH) /* Unreachable interrupt,to read unreachable information */
    {
    }
    if (init_status & GINT_STAT_IP_CONFLI) /* IP address conflict interrupted,advice to change the IP address of the CH392 again and initialize the CH392 */
    {
    }
    if (init_status & GINT_STAT_PHY_CHANGE) /* PHY change interrupt */
    {
        printf("Init status : GINT_STAT_PHY_CHANGE\r\n");
    }
    if (init_status & GINT_STAT_DHCP)
    {
    }
    if (init_status & GINT_STAT_SOCK0)
    {
        CH392SocketInterrupt(0); /* socket 0 interrupt */
    }
    if (init_status & GINT_STAT_SOCK1)
    {
        CH392SocketInterrupt(1); /* socket 1 interrupt */
    }
    if (init_status & GINT_STAT_SOCK2)
    {
        CH392SocketInterrupt(2); /* socket 2 interrupt */
    }
    if (init_status & GINT_STAT_SOCK3)
    {
        CH392SocketInterrupt(3); /* socket 3 interrupt */
    }
#if CHIP == CH392T /* The 392T uses eight sockets by default and a maximum of 32 sockets. For details, see the 392 manual */
    if (init_status & GINT_STAT_SOCK4)
    {
        CH392SocketInterrupt(4); /* socket 4 interrupt */
    }
    if (init_status & GINT_STAT_SOCK5)
    {
        CH392SocketInterrupt(5); /* socket 5 interrupt */
    }
    if (init_status & GINT_STAT_SOCK6)
    {
        CH392SocketInterrupt(6); /* socket 6 interrupt */
    }
    if (init_status & GINT_STAT_SOCK7)
    {
        CH392SocketInterrupt(7); /* socket 7 interrupt */
    }
#endif
}

/**********************************************************************************
 * Function Name  : CH392Init
 * Description    : Configure IP, GWIP, and MAC for the CH392, and initialize the CH392
 * Input          : None
 * Output         : None
 * Return         : Function execution result
 **********************************************************************************/
UINT8 CH392Init(void)
{
    UINT8 i;

    i = CH392CMDCheckExist(0x65);
    if (i != 0x9a)
        return CH392_ERR_UNKNOW;            /* Test cmd , return 0XFA if it fails*/
                                            /* The value 0XFA is usually a hardware error or incorrect read/write timing */
    CH392CMDSetIPAddr(CH392Inf.IPAddr);     /* Set CH392 IP */
    CH392CMDSetGWIPAddr(CH392Inf.GWIPAddr); /* Set GW IP */
    CH392CMDSetMASKAddr(CH392Inf.MASKAddr); /* Set MASK */
    i = CH392CMDInitCH392();                /* initialize the CH392 */
    return i;
}

/**********************************************************************************
 * Function Name  : Gpio_Init
 * Description    : initialize GPIO
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void Gpio_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    // 392Interrupt
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    // 392Rset
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    UINT8 i;
    Delay_Init();
    USART_Printf_Init(115200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
    CH392_UARTPort_Init();
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
    CH392_SPIPort_Init();
#endif

    Gpio_Init();
    CH392Reset();
    Delay_Ms(200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE)
    CH392SetUartBaudRate(921600);
#endif
    printf("CH392EVT Test Demo\r\n");
    i = CH392CMDGetVer();

    printf("CH392VER : %2x\r\n", (UINT16)i);
    InitCH392InfParam(); /* Initialize CH392Inf parameters */
    i = CH392Init();     /* Initialize the CH392 */
    mStopIfError(i);
    while (1)
    {                                              /* Wait for the CH392 Connect Ethernet*/
        if (CH392CMDGetPHYStatus() == PHY_DISCONN) /* Example Query whether the CH392 is connected */
        {
            Delay_Ms(200); /* If no, wait for 200MS and query again */
        }
        else
        {
            printf("CH392 Connect Ethernet\r\n"); /*When the CH392 is connected to the Ethernet, an interruption occurs */
            break;
        }
    }
    InitSocketParam(); /* Initialize socket related variables */
    CH392SocketInitOpen();

    while (1)
    {
        if (Query392Interrupt() == 0)
        {
            CH392GlobalInterrupt();
        }
    }
}
